@@ -13,12 +13,12 @@
 #' @examples
 #' \dontrun{
 #' data("Reading")
-#' FB <- flexreg(accuracy ~ iq, Reading, type="FB", n.iter=1000)
+#' FB <- flexreg(accuracy.adj ~ iq, data = Reading, type="FB", n.iter=1000)
 #' WAIC(FB)
 #'}
 #'
 #' @references {
-#' Vehtari, A., Gelman, A., and Gabry, J. (2017a). Practical Bayesian model evaluation using leave-one-out cross-validation and WAIC. Statistics and Computing. \bold{27}(5), 1413–1432. doi:10.1007/s11222-016-9696-4 \cr
+#' Vehtari, A., Gelman, A., Gabry, J. (2017). Practical Bayesian model evaluation using leave-one-out cross-validation and WAIC. Statistics and Computing. \bold{27}(5), 1413--1432. doi:10.1007/s11222-016-9696-4 \cr
 #' \cr
 #'
 #' }
@@ -30,12 +30,12 @@
 
 WAIC <- function(model, ...){
   x <- model
-  if(class(x) != "flexreg"){
-  if(any(unlist(lapply(x, function(x) class(x) != "flexreg"))))
+  if(!inherits(x, "flexreg")){
+  if(any(unlist(lapply(x, function(x) !inherits(x, "flexreg")))))
     stop("The argument must be an object (or a list of objects) of class `flexreg`")
   }
 
-  if(class(x) == "flexreg"){
+  if(inherits(x, "flexreg")){
     waic_out <- waic(extract_log_lik(x$model))
     loo_out <- loo(extract_log_lik(x$model))
   } else {
